#! /usr/bin/env python
# -*- coding: UTF-8 -*-

from gi.repository import Gtk, Gdk
import os
import random

audio_str = """<ui>
    <menubar name="MenuBar">
        <menu action="File">
            <menu action="sound">
            {0}
            </menu>
        </menu>
    </menubar>
    <popup name="pop_main">
        <menu action="sound">
        {0}
        </menu>
    </popup>
</ui>
"""

ui_str = """<ui>
    <menubar name="MenuBar">
        <menu action="File">
            <menuitem action="open"/>
            <separator/>
            <menuitem action="play"/>
            <menuitem action="stop"/>
            <menuitem action="prev"/>
            <menuitem action="next"/>
            <separator/>
            <menu action="Rep">
                <menuitem action="rep_non"/>
                <menuitem action="rep_one"/>
                <menuitem action="rep_all"/>
                <menuitem action="rep_rdm"/>
            </menu>
            <menu action="asp">
                <menuitem action="asp0"/>
                <menuitem action="asp1"/>
                <menuitem action="asp2"/>
                <menuitem action="asp3"/>
            </menu>
            <menu action="speed">
                <menuitem action="speed_up"/>
                <menuitem action="speed_down"/>
                <menuitem action="speed_reset"/>
            </menu>
            <menu action="sound">
            </menu>
            <separator/>
            <menuitem action="quit"/>
        </menu>
        <menu action="View">
            <menuitem action="full"/>
            <separator/>
            <menuitem action="x00"/>
            <menuitem action="x10"/>
            <menuitem action="x20"/>
            <menuitem action="x30"/>
            <menuitem action="x05"/>
            <menuitem action="x15"/>
            <menuitem action="x25"/>
            <separator/>
            <menuitem action="xs1"/>
            <menuitem action="xs2"/>
            <menuitem action="xs3"/>
        </menu>
        <menu action="Setting">
            <menuitem action="set"/>
        </menu>
        <menu action="Help">
            <menuitem action="online"/>
            <separator/>
            <menuitem action="ver"/>
        </menu>
    </menubar>
    <popup name="pop_main">
        <menuitem action="open"/>
        <separator/>
        <menuitem action="play"/>
        <menuitem action="stop"/>
        <menuitem action="prev"/>
        <menuitem action="next"/>
        <separator/>
        <menu action="View">
            <menuitem action="full"/>
            <separator/>
            <menuitem action="x00"/>
            <menuitem action="x10"/>
            <menuitem action="x20"/>
            <menuitem action="x30"/>
            <menuitem action="x05"/>
            <menuitem action="x15"/>
            <menuitem action="x25"/>
            <separator/>
            <menuitem action="xs1"/>
            <menuitem action="xs2"/>
            <menuitem action="xs3"/>
        </menu>
        <menu action="Rep">
            <menuitem action="rep_non"/>
            <menuitem action="rep_one"/>
            <menuitem action="rep_all"/>
            <menuitem action="rep_rdm"/>
        </menu>
        <menu action="asp">
            <menuitem action="asp0"/>
            <menuitem action="asp1"/>
            <menuitem action="asp2"/>
            <menuitem action="asp3"/>
        </menu>
        <menu action="speed">
            <menuitem action="speed_up"/>
            <menuitem action="speed_down"/>
            <menuitem action="speed_reset"/>
        </menu>
        <menu action="sound">
        </menu>
        <separator/>
        <menuitem action="set"/>
        <separator/>
        <menuitem action="quit"/>
    </popup>
    <popup name="pop_list">
        <menuitem action="list_close"/>
    </popup>
</ui>"""

size_str = ["win","x1.0","x2.0","x3.0","x0.5","x1.5","x2.5","set1","set2","set3"]
ac1_str = ["x00","x10","x20","x30","x05","x15","x25","xs1","xs2","xs3"]
ac2_str = ["rep_non", "rep_one", "rep_all", "rep_rdm"]
ac3_str = ["asp0", "asp1", "asp2", "asp3"]
asp_str = ["default", "4:3", "16:9", "custom"]
MIMES = "video/x-ms-wmv|video/x-flv|video/mp4|video/mpeg|video/x-msvideo"
playback_speed = (0.12, 0.25, 0.33, 0.5, 0.67, 1.0, 1.5, 2.0, 3.0, 4.0, 8.0)

#
# StatusBar
#

class CStatusBar(Gtk.Box):
    """
        Instead GtkStatusbar
    """
    def __init__(self, owner, arg):
        """
            @param arg: list of string
              labels Initial value
        """
        self._owner = owner
        # public menber
        self.label = []
        # init Original inheritance
        Gtk.Box.__init__(self)
        # GtkLabel and GtkVSeparator
        hb = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        for s in arg:
            l = Gtk.Label(s)
            p = Gtk.VSeparator()
            hb.pack_start(l, False, False, 10)
            hb.pack_start(p, False, False, 0)
            self.label.append(l)
        # Dummy
        ld = Gtk.Label()
        hb.pack_start(ld, True, True, 0)
        # Grip
        grip = Gtk.DrawingArea()
        grip.set_size_request(16, 16)
        grip.set_events(Gdk.EventMask.EXPOSURE_MASK | Gdk.EventMask.BUTTON_PRESS_MASK)
        grip.connect("draw", self.__on_grip_draw)
        grip.connect("button-press-event", self.__on_resize)
        grip.connect("realize", self.__on_realize)
        hb.pack_end(grip, False, False, 0)
        # pack
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        s0 = Gtk.HSeparator()
        vbox.pack_start(s0, False, False, 0)
        vbox.pack_start(hb, False, False, 0)
        self.pack_start(vbox, True, True, 0)
        self.show_all()

    def __on_resize(self, widget, event):
        """
            begin resize @ Toplevel Window
        """
        #self._owner.hand_resize()
        self._owner.begin_resize_drag(
                Gdk.WindowEdge.SOUTH_EAST,
                event.button,
                int(event.x_root),
                int(event.y_root),
                event.time)

    def __on_realize(self, widget, event=None):
        """
            cursor change
        """
        cur_grip = Gdk.Cursor(Gdk.CursorType.BOTTOM_RIGHT_CORNER)
        widget.props.window.set_cursor(cur_grip)

    def __on_grip_draw(self, widget, cr):
        """
            Draw Resize Grip
        """
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        Gtk.paint_resize_grip(
                widget.props.style,
                cr,
                Gtk.StateType.NORMAL,
                None,
                None,
                Gdk.WindowEdge.SOUTH_EAST,
                width - 16,
                height - 16,
                16,
                16 )

#
# ListBox
#

class CListBox(Gtk.Box):
    """
        GtkTreeView wrapping panels
    """
    def __init__(self, window):
        """
            @param window: Toplevel Window
            Synchronize Resize
        """
        self._win = window
        Gtk.Box.__init__(self)#, Gtk.Orientation.HORIZONTAL, 0)
        # private:
        self.__resize_w = 0
        self.__resize_x = 0
        self.__resize_y = 0
        self.__cx = 0
        self.__cy = 0
        # partition
        vs = Gtk.Separator.new(Gtk.Orientation.VERTICAL)
        vs.set_size_request(5, -1)
        self.pack_start(vs, False, True, 0)
        # on ScrolledWindow TreeView
        self.sw = Gtk.ScrolledWindow()
        self.sw.set_size_request(100, -1)
        self.sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.sw.set_shadow_type(Gtk.ShadowType.IN)
        # GtkTreeView
        self.view = Gtk.TreeView()
        self.sw.add(self.view)
        # GtkTreeColumn
        self.column1 = Gtk.TreeViewColumn()
        self.view.append_column(self.column1)
        self.connect("check-resize", self.__on_size_request)
        cell1 = Gtk.CellRendererText()
        self.column1.pack_start(cell1, 0)
        self.column1.add_attribute(cell1, "text", 0)
        # liststore create  
        self.liststore = Gtk.ListStore(str)
        self.view.set_model(self.liststore)
        self.view.set_headers_visible(False)
        self.selection = self.view.get_selection()
        self.selection.set_mode(Gtk.SelectionMode.SINGLE)
        # treeview pack
        self.pack_start(self.sw, True, True, 0)
        # DrawingArea
        self.dwa = Gtk.DrawingArea()
        self.dwa.set_events(Gdk.EventMask.EXPOSURE_MASK | 
                            Gdk.EventMask.POINTER_MOTION_MASK | 
                            Gdk.EventMask.POINTER_MOTION_HINT_MASK | 
                            Gdk.EventMask.BUTTON_PRESS_MASK | 
                            Gdk.EventMask.BUTTON_RELEASE_MASK | 
                            Gdk.EventMask.STRUCTURE_MASK )
                            #Gdk.EventMask.VISIBILITY_NOTIFY_MASK )
        self.dwa.set_size_request(10, -1)
        self.pack_end(self.dwa, False, True, 0)
        # Connect
        self.dwa.connect("button-press-event", self.__on_button_press_event)
        self.dwa.connect("motion-notify-event", self.__on_motion_notify_event)
        self.dwa.connect("button-release-event", self.__on_button_release_event)
        self.dwa.connect("draw", self.__on_draw_event)
        self.dwa.connect("realize", self.__on_realize)
        #self.liststore.set_sort_column_id(0, Gtk.SORT_ASCENDING)
        self.show_all()

    def is_text(self, text):
        return text in [ r[0] for r in self.liststore ]

    def select_text(self, text):
        model = self.view.get_model()
        it = model.get_iter_first()
        if it == None:
            self.selection.unselect_all()
        else:
            self._n = 0
            while 1:
                s = model.get_value(it, 0)
                if s == text:
                    self.selection.select_iter(it)
                    break
                it = model.iter_next(it)
                if it == None: break

    def selection_scroll(self):
        """
            Auto Scroll
        """
        # Get GtkAdjustment
        adjustment = self.view.get_vadjustment()
        # Is Select Check
        n = self.get_select_num()
        if n == -1:
            return
        page_size = adjustment.props.page_size
        column_height = self.column1.cell_get_size(None)[3]
        # calc
        sc = column_height * n
        # into Rect
        v = adjustment.get_value()
        h = page_size + v - (column_height * 2)
        if sc == 0:
            adjustment.set_value(0.0)
        elif sc < v:
            adjustment.set_value(sc - column_height)
        elif sc >= h:
            adjustment.set_value(sc - page_size + (column_height * 2))

    def get_select_num(self):
        model, it = self.selection.get_selected()
        if it:
            return model.get_path(it)[0]
        return -1

    def __on_size_request(self, widget, event=None):
        """
            Select Item View
        """
        self.selection_scroll()

    def __on_button_press_event(self, widget, event):
        # begin resize
        if event.button == 1:
            self.__resize_w = self.get_property("width_request")
            self.__resize_x = event.x_root
            self.__cx, self.__cy = self._win.get_size()

    def __on_motion_notify_event(self, widget, event):
        # size change
        if self.__resize_w:
            x = event.x_root - self.__resize_x
            if self.__resize_w + x > 0:
                self.props.width_request = self.__resize_w + x
                self._win.resize(int(self.__cx + x), int(self.__cy))

    def __on_button_release_event(self, widget, event):
        # end resize
        if event.button == 1:
            self.__resize_w = 0
            self.dwa.queue_draw()

    def __on_draw_event(self, widget, cr):
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        cr.set_source_rgb(1.0, 1.0, 1.0)
        cr.paint()
        cr.set_source_rgb(0.0, 0.0, 0.0)
        cr.set_line_width(0.2)
        for i in range(10):
            if i%3 == 0:
                cr.move_to(i, 0)
                cr.line_to(i, height)
                cr.stroke()

    def __on_realize(self, widget, event=None):
        cur_grip = Gdk.Cursor(Gdk.CursorType.RIGHT_TEE)
        self.dwa.props.window.set_cursor(cur_grip)


#
# ToolBox
#

class CToolBox(Gtk.Box):
    """
        ToolBox
    """
    def __init__(self):
        Gtk.Box.__init__(self)
        # SeekBar
        self.seekadj = Gtk.Adjustment(0.0, 0.0, 100.0, 1.0, 1.0, 1.0)
        self.seekbar = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, self.seekadj)
        self.seekbar.set_events(
                Gdk.EventMask.BUTTON_PRESS_MASK | 
                Gdk.EventMask.BUTTON_RELEASE_MASK | 
                Gdk.EventMask.STRUCTURE_MASK )
        self.seekbar.set_draw_value(False)
        # ctrl pack
        self.hbox = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        # images
        self._path = os.path.dirname( __file__ )
        self.img = []
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/pausebutton.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/stopbutton.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/playlist.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/rep_one.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/rep_all.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/rep_rdm.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/pausebutton.xpm")))
        self.img.append(Gtk.Image.new_from_file(os.path.join(self._path, "img/playbutton.xpm")))
        # button
        self.__button = []
        for i in range(6):
            if i < 3:
                b = Gtk.Button()
            else:
                b = Gtk.ToggleButton()
            b.set_focus_on_click(False)   # no focus
            b.set_relief(Gtk.ReliefStyle.NONE) # Flat Button
            b.add(self.img[i])
            self.__button.append(b)
        self.play = self.__button[0]
        self.stop = self.__button[1]
        self.playlist = self.__button[2]
        self.one = self.__button[3]
        self.all = self.__button[4]
        self.rdm = self.__button[5]
        # volume bar
        self.volumeadj = Gtk.Adjustment(50.0, 0.0, 101.0, -1.0, 1.0, 1.0)
        self.volumebar = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, self.volumeadj)
        #self.volumebar = Gtk.HScale(self.volumeadj)
        self.volumebar.set_value_pos(Gtk.PositionType.RIGHT)
        self.volumebar.set_size_request(100, -1)
        self.volumebar.set_digits(0)
        self.volumebar.set_show_fill_level(True)
        self.volumebar.set_events(  Gdk.EventMask.BUTTON_PRESS_MASK | 
                                    Gdk.EventMask.BUTTON_RELEASE_MASK | 
                                    Gdk.EventMask.STRUCTURE_MASK )
        # packing
        self.hbox.pack_start(self.play, False, False, 0)
        self.hbox.pack_start(self.stop, False, False, 0)
        self.hbox.pack_start(self.volumebar, False, False, 0)
        self.hbox.pack_end(self.playlist, False, False, 0)
        self.hbox.pack_end(self.rdm, False, False, 0)
        self.hbox.pack_end(self.all, False, False, 0)
        self.hbox.pack_end(self.one, False, False, 0)
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        vbox.pack_start(self.seekbar, True, True, 0)
        vbox.pack_start(self.hbox, False, False, 0)
        self.pack_start(vbox, True, True, 0)
        self.num = 0
        #self.set_sensitive(False)
        self.set_enable_ctrl(False)
        self.show_all()

    def change_img(self, num):
        if self.num == num:
            return
        if num == 1:
            self.img[0].set_from_pixbuf(self.img[7].get_pixbuf())
        else:
            self.img[0].set_from_pixbuf(self.img[6].get_pixbuf())
        self.num = num

    def set_enable_ctrl(self, b):
        self.play.set_sensitive(b)
        self.stop.set_sensitive(b)
        self.seekbar.set_sensitive(b)


class CRList():
    """
        Random File List
    """
    def __init__(self):
        self.l = []
        self.m = []

    def get(self):
        for s in self.l:
            if not s in self.m:
                self.m.append(s)
                return s
        self.m[:] = []
        return self.l[0]

    def append(self, text):
        self.l.append(text)

    def clear(self):
        self.l[:] = []
        self.m[:] = []

    def shuffle(self):
        random.shuffle(self.l)


class CFullCtrl(Gtk.Window):
    """
        Full Screen Controrer
    """
    def __init__(self, parent, box, parent2, bar):
        """
            GtkWindow at GTK_WINDOW_POPUP
        """
        Gtk.Window.__init__(self, Gtk.WindowType.POPUP)
        # memory pointer
        self.p1 = parent
        self.p2 = parent2
        self.box = box
        self.bar = bar
        # Get Desktop Window Geometry
        root = Gdk.get_default_root_window()
        x, y, w, h = root.get_geometry()
        # move and resize
        height = box.get_allocated_height() + bar.get_allocated_height()
        self.set_size_request(w, height)
        self.y_pos = h - height
        self.move(0, self.y_pos)
        # Change parent
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.box.reparent(vbox)
        self.bar.reparent(vbox)
        vbox.show_all()
        self.add(vbox)
        self.connect("destroy", self.on_destroy)

    def on_destroy(self, widget, event=None):
        """
            Return Parent
        """
        self.box.reparent(self.p1)
        self.p1.set_child_packing(self.box, False, True, 0, Gtk.PackType.END)
        self.bar.reparent(self.p2)
        self.p2.set_child_packing(self.bar, False, False, 0, Gtk.PackType.START)
        return True


class CPath(object):
    __slots__ = ["dirc", "name"]
    def __init__(self):
        self.dirc = ""
        self.name = ""

    def set_path(self, fullpath):
        self.dirc, self.name = os.path.split(fullpath)


class AudioMenu(object):
    __slots__ = ["uimanager", "ui_id", "action_group"]
    def __init__(self, uimanager):
        self.uimanager = uimanager
        self.ui_id = -1

    def add_menu(self, actions, xml, callback, value):
        self.action_group = Gtk.ActionGroup("AudioMenuActions")
        self.action_group.add_radio_actions(actions, value, callback)
        self.uimanager.insert_action_group(self.action_group, -1)
        self.ui_id = self.uimanager.add_ui_from_string(xml)

    def remove_menu(self):
        if self.ui_id != -1:
            self.uimanager.remove_ui(self.ui_id)
            self.uimanager.remove_action_group(self.action_group)
            self.uimanager.ensure_update()
            self.ui_id = -1
            del self.action_group

#
# Structs (Windows compatible)
#

class CSize(object):
    __slots__ = ["width", "height"]
    def __init__(self, w=0, h=0):
        self.width = w
        self.height = h

class CPoint(object):
    __slots__ = ["x", "y"]
    def __init__(self, x=0, y=0):
        self.x = x
        self.y = y

class CGesture(object):
    __slots__ = ["start", "result", "title", "prevchar", "x", "y"]
    def __init__(self):
        self.start = False
        self.result = ""
        self.title = ""
        self.prevchar = ""
        self.x = 0
        self.y = 0

class CSetting(object):
    __slots__ = [
            "mimes", "position", "init_size_on", "init_size_val", "esc_exit",
            "mem_rep", "mem_aspect", "endressloop", "severity", "aspect_rate",
            "set_size", "size_win_only"]
    def __init__(self):
        self.mimes = MIMES
        self.position = False
        self.init_size_on = False
        self.init_size_val = 0
        self.esc_exit = False
        self.mem_rep = False
        self.mem_aspect = False
        self.endressloop = False
        self.severity = 25
        self.aspect_rate = [1,1]
        self.set_size = [[320,240],[640,480],[1280,720]]
        self.size_win_only = False

class CSettingWin(object):
    __slots__ = [
            "list_width", "list_show", "volume", "video_size", "repeat",
            "aspect", "repeat_mem", "aspect_mem", "n_audio", "speed"]
    def __init__(self):
        self.list_width = 100
        self.list_show = True
        self.volume = 100.0
        self.video_size = 0
        self.repeat = 0
        self.aspect = 0
        self.repeat_mem = 0
        self.aspect_mem = 0
        self.n_audio = 0
        self.speed = 5

class SeekInfo(object):
    __slots__ = ["is_seek", "playing", "button"]
    def __init__(self):
        self.is_seek = False
        self.playing = False
        self.button  = 0

if __name__ == "__main__":
    # Unconscientious debug
    import y901main
    y901main.main()
