#!/usr/bin/env python
#-*- coding:utf-8 -*-

import sys
import gi
try:
    gi.require_version("Gtk", "3.0")
    gi.require_version("Gst", "1.0")
except Exception, e:
    print "Error: {0}".format(e)
    sys.exit()

from gi.repository import Gtk, Gst, GstAudio, GLib, Gdk, GdkPixbuf, GObject, Gio, GdkX11, GstVideo

import os
import time
#
import inifile8
import y901dialog
from common import *
import numsort

VERSION = "0.99.2"

class Y901window(Gtk.Window):
    """
        Main Window Class
    """
    def __init__(self):
        """
            Create Y901x Window
        """
        sys.path.insert(0, GLib.path_get_dirname(__file__))
        Gtk.Window.__init__(self)
        # attribute
        self.uri = None
        self.is_playing = False
        self.path = CPath()
        self.seekinfo = SeekInfo()
        self.timer_id = -1
        self.p_position = Gst.CLOCK_TIME_NONE
        self.p_duration = Gst.CLOCK_TIME_NONE
        self.rlist = CRList()
        self.fullscreen = False
        self.fullsize = CSize(320, 240)
        self.fullobj = None
        self.full_list = False
        self.settingwin = CSettingWin()
        self.setting = CSetting()
        self.mime_list = []
        self.gesture = CGesture()
        self.vsize = CSize(1, 1)
        self.mousepoint = CPoint()
        self.invisiblecount = 0
        # Read config file
        inipath = "{0}/sasakima/y901x.conf".format(GLib.get_user_config_dir())
        if GLib.file_test(inipath, GLib.FileTest.EXISTS):
            ini = inifile8.Inifile(inipath)
            try:
                # [window]
                x = ini.read_int("window", "left", 0)
                y = ini.read_int("window", "top", 0)
                cx = ini.read_int("window", "width", 320)
                cy = ini.read_int("window", "height", 320)
                self.settingwin.list_width  = ini.read_int("window", "list_width", 100)
                self.settingwin.list_show   = ini.read_bool("window", "list_show", True)
                self.settingwin.volume      = ini.read_float("window", "volume", 100.0)
                self.settingwin.video_size  = ini.read_int("window", "video_size", 0)
                self.settingwin.repeat_mem  = ini.read_int("window", "repeat", 0)
                self.settingwin.aspect_mem  = ini.read_int("window", "aspect", 0)
                self.settingwin.n_audio     = ini.read_int("window", "n_audio", 0)
                # [setting]
                self.setting.position       = ini.read_bool("setting", "position", False)
                self.setting.esc_exit       = ini.read_bool("setting", "esc_exit", False)
                self.setting.mem_rep        = ini.read_bool("setting", "mem_rep", False)
                self.setting.mem_aspect     = ini.read_bool("setting", "mem_aspect", False)
                self.setting.endressloop    = ini.read_bool("setting", "loop", False)
                self.setting.size_win_only  = ini.read_bool("setting", "size_win_only", False)
                self.setting.init_size_on   = ini.read_bool("setting", "init_size_on", False)
                self.setting.init_size_val  = ini.read_int("setting", "init_size_val", 0)
                self.setting.severity       = ini.read_int("setting", "severity", 25)
                self.setting.mimes          = ini.read_str("setting", "mimes", MIMES)
                self.setting.set_size[0][0] = ini.read_int("setting", "set_size1", 320)
                self.setting.set_size[0][1] = ini.read_int("setting", "set_size2", 240)
                self.setting.set_size[1][0] = ini.read_int("setting", "set_size3", 640)
                self.setting.set_size[1][1] = ini.read_int("setting", "set_size4", 480)
                self.setting.set_size[2][0] = ini.read_int("setting", "set_size5", 1280)
                self.setting.set_size[2][1] = ini.read_int("setting", "set_size6", 720)
                self.setting.aspect_rate[0] = ini.read_int("setting", "aspect_rate0", 1)
                self.setting.aspect_rate[1] = ini.read_int("setting", "aspect_rate1", 1)
            except Exception, s:
                self.messagebox("{0}".format(s))
                sys.exit()
            # resize and move
            if self.setting.init_size_on:
                self.settingwin.video_size = self.setting.init_size_val
            self.resize(cx, cy)
            if self.setting.position:
                self.move(x, y)
        # List up the MIME Types
        self.mime_list = self.setting.mimes.split("|")
        # Create GtkUIManager
        self.uimanager = Gtk.UIManager()
        self.audiomenu = AudioMenu(self.uimanager)
        # GtkAccelGroup (accel_key, accel_mods, accel_flags, callback)
        self.accelgroup0 = self.uimanager.get_accel_group()
        self.accelgroup0.connect(Gdk.KEY_KP_0, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_1, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_2, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_3, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_4, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_5, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_6, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_7, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_8, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_9, 0, Gtk.AccelFlags.VISIBLE, self.on_keypad)
        self.accelgroup0.connect(Gdk.KEY_KP_Add, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_KP_Subtract, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_KP_Enter, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        #
        self.accelgroup0.connect(Gdk.KEY_F12, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_F11, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_Return, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_F5, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_F2, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_Escape, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_Delete, 0, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_Delete, Gdk.ModifierType.SHIFT_MASK, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.accelgroup0.connect(Gdk.KEY_Delete, Gdk.ModifierType.CONTROL_MASK, Gtk.AccelFlags.VISIBLE, self.on_keydown)
        self.add_accel_group(self.accelgroup0)
        # Create GtkActionGroup
        self.actiongroup = Gtk.ActionGroup("sasakimamenu")
        # GtkActionEntry (name, stock_id, label, accelerator, tooltip, callback)
        # (warning: Up, Down key is Don't work)
        self.ac0 = [("open", Gtk.STOCK_OPEN, "開く(_O)", "<Control>O", "開く", self.on_open),
                    ("play", Gtk.STOCK_MEDIA_PLAY, "再生/ポーズ(_P)", "space", "再生/ポーズ", self.on_play),
                    ("stop", Gtk.STOCK_MEDIA_STOP, "停止(_S)", "BackSpace", "停止", self.on_stop),
                    ("prev", Gtk.STOCK_GO_UP, "手前を再生(_U)", "Up", "手前を再生", self.on_prev),
                    ("next", Gtk.STOCK_GO_DOWN, "次を再生(_D)", "Down", "次を再生", self.on_next),
                    ("speed_up", None, "再生速度を上げる(_F)", "plus", "再生速度を上げる", self.on_speed_up),
                    ("speed_down", None, "再生速度を下げる(_W)", "minus", "再生速度を下げる", self.on_speed_down),
                    ("speed_reset", None, "再生速度を元に戻す(_R)", "equal", "再生速度を元に戻す", self.on_speed_reset),
                    ("quit", Gtk.STOCK_QUIT, "終了(_Q)", "<Control>Q", "さいなら", self.on_quit),
                    ("set", Gtk.STOCK_PREFERENCES, "設定(_P)", "P", "設定", self.on_setting),
                    ("ver", Gtk.STOCK_ABOUT, "情報(_B)", "<Control>B", "情報", self.on_about),
                    ("full", None, "フルスクリーン(_F)", "Z", "フルスクリーン切り替え", self.on_full),
                    ("online", Gtk.STOCK_HELP, "オンラインヘルプを開く...", None, "オンラインヘルプを開く...", self.on_help),
                    ("File", None, "ファイル(_F)"),
                    ("Rep", None, "リピート(_R)"),
                    ("asp", None, "アスペクト比(_A)"),
                    ("speed", None, "再生速度(_S)"),
                    ("sound", None, "音声(_U)"),
                    ("View", None, "表示(_V)"),
                    ("Setting", None, "設定(_P)"),
                    ("Help", None, "ヘルプ(_H)")]
        self.actiongroup.add_actions(self.ac0)
        self.ac4 = [("list_close", None, "リストを閉じる", None, "リストを閉じる", self.on_list_button_clicked)]
        self.actiongroup.add_actions(self.ac4)
        # GtkRadioActionEntry (name, stock_id, label, accelerator, tooltip, value)
        self.ac1 = [("x00", None, "ウインドウサイズ", "0", "ウインドサイズ", 0),
                    ("x10", None, "x1.0", "1", "x1.0", 1),
                    ("x20", None, "x2.0", "2", "x2.0", 2),
                    ("x30", None, "x3.0", "3", "x3.0", 3),
                    ("x05", None, "x0.5", "4", "x0.5", 4),
                    ("x15", None, "x1.5", "5", "x1.5", 5),
                    ("x25", None, "x2.5", "6", "x2.5", 6),
                    ("xs1", None, "set1", "7", "set1", 7),
                    ("xs2", None, "set2", "8", "set1", 8),
                    ("xs3", None, "set3", "9", "set1", 9) ]
        self.ac2 = [("rep_non", None, "リピート無し", "F6", "リピート無し", 0),
                    ("rep_one", None, "シングルリピート", "F7", "シングルリピート", 1),
                    ("rep_all", None, "すべてリピート", "F8", "すべてリピート", 2),
                    ("rep_rdm", None, "ランダム再生", "F9", "ランダム再生", 3) ]
        self.ac3 = [("asp0", None, "default", "<Control>A", "アスペクト比 default", 0),
                    ("asp1", None, "4:3", "<Control>S", "アスペクト比 4:3", 1),
                    ("asp2", None, "16:9", "<Control>D", "アスペクト比 16:9", 2),
                    ("asp3", None, "custom", "<Control>Z", "アスペクト比 custom", 3) ]
        # GtkRadioAction (entries, value=0, on_change=None, user_data=None)
        self.actiongroup.add_radio_actions(self.ac1, -1, self.on_size_change)
        self.actiongroup.add_radio_actions(self.ac2, 0, self.on_rep_change)
        self.actiongroup.add_radio_actions(self.ac3, 0, self.on_asp_change)
        # Update the GtkUIManager
        self.uimanager.insert_action_group(self.actiongroup, 0)
        self.uimanager.add_ui_from_string(ui_str)
        self.uimanager.ensure_update()
        # Get the PopupMenu and Next menu
        self.popup_menu = self.uimanager.get_widget('/pop_main')
        self.next_menu = self.uimanager.get_widget('/MenuBar/File/next')
        # StatusBar
        self.statusbar = CStatusBar(self, ["00:00/00:00", "win", "defaule", "0.00x"])
        # ListBox
        self.listbox = CListBox(self)
        self.listbox.set_size_request(self.settingwin.list_width, -1)
        self.listbox.view.connect("button-release-event", self.on_listbox_button_release)
        self.listbox.view.connect("button-press-event", self.on_listbox_button_press)
        self.listbox.props.visible = self.settingwin.list_show
        self.listpopup = self.uimanager.get_widget('/pop_list')
        # ToolBox
        self.toolbox = CToolBox()
        self.toolbox.play.connect("clicked", self.on_play)
        self.toolbox.stop.connect("clicked", self.on_stop)
        self.toolbox.playlist.connect("clicked", self.on_list_button_clicked)
        self.toolbox.seekbar.connect("value-changed", self.on_seek_value_changed)
        self.toolbox.seekbar.connect("button-press-event", self.on_seek_button_press_event)
        self.toolbox.seekbar.connect("button-release-event", self.on_seek_button_release_event)
        self.toolbox.volumebar.connect("button-release-event", self.on_volume_button_release_event)
        self.toolbox.one.connect("clicked", self.on_rep_btn, 1)
        self.toolbox.all.connect("clicked", self.on_rep_btn, 2)
        self.toolbox.rdm.connect("clicked", self.on_rep_btn, 3)
        self.toolbox.volumeadj.set_value(self.settingwin.volume)
        self.toolbox.volumebar.props.sensitive = False
        # PlayBin2
        self.player = Gst.ElementFactory.make("playbin", None)
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.gst_on_message)
        bus.connect("sync-message::element", self.gst_on_sync_message)
        # Container
        vbox_submain = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 0)
        vbox_submain.show()
        self.hbox_ctrl = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.hbox_ctrl.show()
        self.vbox_main = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        self.vbox_main.show()
        # Video Window, Background Color to Black
        self.video_window = Gtk.DrawingArea()
        self.video_window.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0,0,0,1))
        self.video_window.props.can_focus = True
        # GtkAspectFrame, draw a Background Color to Black
        self.aframe = Gtk.AspectFrame.new("null", 0.5, 0.5, 4.0/3.0, False)
        #self.aframe.set_label_align(0.0, 0.0)
        self.aframe.props.label = None
        self.aframe.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0,0,0,1))
        self.aframe.set_shadow_type(Gtk.ShadowType.NONE)
        self.aframe.set_events(
                Gdk.EventMask.EXPOSURE_MASK |
                Gdk.EventMask.STRUCTURE_MASK |
                Gdk.EventMask.VISIBILITY_NOTIFY_MASK)
        self.aframe.connect("draw", self.on_aframe_draw)
        self.aframe.add(self.video_window)
        # menubar
        self.menubar = self.uimanager.get_widget("/MenuBar")
        # packing
        self.hbox_ctrl.pack_start(self.menubar, False, True, 0)
        self.hbox_ctrl.pack_start(self.aframe, True, True, 0)
        self.hbox_ctrl.pack_end(self.toolbox, False, True, 0)
        self.hbox_ctrl.show_all()
        vbox_submain.pack_start(self.hbox_ctrl, True, True, 0)
        vbox_submain.pack_end(self.listbox, False, True, 0)
        self.vbox_main.pack_start(vbox_submain, True, True, 0)
        self.vbox_main.pack_start(self.statusbar, False, False, 0)
        self.add(self.vbox_main)
        # TitleBar
        iconpath = GLib.build_filenamev([GLib.path_get_dirname(__file__), "y901x.xpm"])
        self.y901icon = GdkPixbuf.Pixbuf.new_from_file(iconpath)
        self.set_icon(self.y901icon)
        self.set_title("Y901x")
        # Drop source set
        dnd_list = Gtk.TargetEntry.new("text/uri-list", 0, 0)
        self.drag_dest_set(
                Gtk.DestDefaults.MOTION |
                Gtk.DestDefaults.HIGHLIGHT |
                Gtk.DestDefaults.DROP,
                [dnd_list],
                Gdk.DragAction.MOVE )
        self.drag_dest_add_uri_targets()
        # GDK event set
        self.set_events(
                Gdk.EventMask.KEY_PRESS_MASK |
                Gdk.EventMask.KEY_RELEASE_MASK |
                Gdk.EventMask.STRUCTURE_MASK )
        self.video_window.set_events(
                Gdk.EventMask.LEAVE_NOTIFY_MASK |
                Gdk.EventMask.BUTTON_PRESS_MASK |
                Gdk.EventMask.BUTTON_RELEASE_MASK |
                Gdk.EventMask.POINTER_MOTION_MASK |
                Gdk.EventMask.POINTER_MOTION_HINT_MASK )
        # Create invisible cursor
        self.noncursor = Gdk.Cursor.new(Gdk.CursorType.BLANK_CURSOR)
        # Signals
        self.video_window.connect("button-press-event", self.on_button_press)
        self.video_window.connect("button-release-event", self.on_button_release)
        self.video_window.connect("motion-notify-event", self.on_motion_notify)
        self.connect("drag-data-received", self.on_drag_data_received)
        self.connect("key-press-event", self.on_key_press)
        self.connect("key-release-event", self.on_key_release)
        self.connect("delete-event", self.on_quit)
        self.show()
        # Under the Show After
        # Radio menu position changes. Throw on_size_change signal.
        action = self.actiongroup.get_action(ac1_str[self.settingwin.video_size])
        action.set_current_value(self.settingwin.video_size)
        # Repert memory Attach
        if self.setting.mem_rep:
            action2 = self.actiongroup.get_action(ac2_str[self.settingwin.repeat_mem])
            action2.set_current_value(self.settingwin.repeat_mem)
        # Aspect rate Attach
        if self.setting.mem_aspect:
            action3 = self.actiongroup.get_action(ac3_str[self.settingwin.aspect_mem])
            action3.set_current_value(self.settingwin.aspect_mem)
        # move focus
        self.set_focus(self.video_window)
        # change gstreamer volume
        self.toolbox.volumebar.connect("value-changed", self.on_volume_value_changed)
        # argv processing is main function

########################################################################
#
#                       Functions
#
########################################################################


    def messagebox(self, text, icon=Gtk.MessageType.WARNING, button=Gtk.ButtonsType.OK):
        """
            Simple MessageBox
        """
        dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, icon, button, text)
        dlg.set_title("from Y901x")
        r = dlg.run()
        dlg.destroy()
        return r

    def messagebox_trash(self, text):
        """
            Trash Only MessageBox
        """
        dlg = Gtk.MessageDialog(self, Gtk.DialogFlags.MODAL, Gtk.MessageType.QUESTION,  Gtk.ButtonsType.NONE, text)
        dlg.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
        dlg.add_button("ゴミ箱へ移動する", Gtk.ResponseType.OK)
        dlg.set_default_response(Gtk.ResponseType.OK)
        dlg.set_title("from Y901x")  
        r = dlg.run()  
        dlg.destroy()
        return r

    def set_uri(self, uri):
        """
            Once NULL.
            Non supported file => gst_on_message
        """
        if uri == None:
            return
        self.clear()
        self.player.set_state(Gst.State.NULL)
        self.toolbox.set_enable_ctrl(False)
        while Gtk.events_pending():
            Gtk.main_iteration()
        self.uri = uri # v3.6
        self.player.set_property("uri", uri)
        self.player.set_state(Gst.State.PLAYING)
        self.set_focus(self.video_window)

    def close(self):
        """
            Application Quit and Setting File Write
        """
        self.player.set_state(Gst.State.NULL)
        #
        # Search "y901x.conf"
        direpath = GLib.get_user_config_dir() + "/sasakima"
        directory = Gio.file_new_for_path(direpath)
        if not directory.query_exists(None):
            directory.make_directory(None)
        confpath = direpath + "/y901x.conf"
        ini = inifile8.Inifile(confpath)
        try:
            # [window]
            x, y = self.get_position()
            ini.write_int("window", "left", x)
            ini.write_int("window", "top", y)
            if self.fullscreen:
                ini.write_int("window", "width", self.fullsize.width)
                ini.write_int("window", "height", self.fullsize.height)
            else:
                cx, cy = self.get_size()
                ini.write_int("window", "width", cx)
                ini.write_int("window", "height", cy)
            ini.write_int("window", "list_width", self.listbox.props.width_request)
            ini.write_bool("window", "list_show", self.listbox.props.visible)
            ini.write_float("window", "volume", self.toolbox.volumeadj.get_value())
            ini.write_int("window", "video_size", self.settingwin.video_size)
            ini.write_int("window", "repeat", self.settingwin.repeat)
            ini.write_int("window", "aspect", self.settingwin.aspect)
            ini.write_int("window", "n_audio", self.settingwin.n_audio)
            # [setting]
            ini.write_bool("setting", "position", self.setting.position)
            ini.write_bool("setting", "esc_exit", self.setting.esc_exit)
            ini.write_bool("setting", "mem_rep", self.setting.mem_rep)
            ini.write_bool("setting", "mem_aspect", self.setting.mem_aspect)
            ini.write_bool("setting", "loop", self.setting.endressloop)
            ini.write_bool("setting", "size_win_only", self.setting.size_win_only)
            ini.write_bool("setting", "init_size_on", self.setting.init_size_on)
            ini.write_int("setting", "init_size_val", self.setting.init_size_val)
            ini.write_int("setting", "severity", self.setting.severity)
            ini.write_str("setting", "mimes", self.setting.mimes)
            ini.write_int("setting", "set_size1", self.setting.set_size[0][0])
            ini.write_int("setting", "set_size2", self.setting.set_size[0][1])
            ini.write_int("setting", "set_size3", self.setting.set_size[1][0])
            ini.write_int("setting", "set_size4", self.setting.set_size[1][1])
            ini.write_int("setting", "set_size5", self.setting.set_size[2][0])
            ini.write_int("setting", "set_size6", self.setting.set_size[2][1])
            ini.write_int("setting", "aspect_rate0", self.setting.aspect_rate[0])
            ini.write_int("setting", "aspect_rate1", self.setting.aspect_rate[1])
            ini.save()
        except Exception, s:
            self.messagebox("{0}".format(s))
        Gtk.main_quit()

    def set_play_speed(self, n):
        """
            http://docs.gstreamer.com/display/GstSDK/Basic+tutorial+13%3A+Playback+speed
        """
        pos = self.player.query_position(Gst.Format.TIME)[1]
        self.settingwin.speed = n
        if playback_speed[n] > 0:
            event = Gst.Event.new_seek(
                    playback_speed[n],
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET,
                    pos,
                    Gst.SeekType.NONE,
                    0)
        else:
            event = Gst.Event.new_seek(
                    playback_speed[n],
                    Gst.Format.TIME,
                    Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                    Gst.SeekType.SET,
                    0,
                    Gst.SeekType.SET,
                    pos)
        self.player.send_event(event)
        self.statusbar.label[3].set_text("{0:.2f}x".format(playback_speed[n]))

    def set_play_position(self, location):
        """
            Media Seeking
            SeekFlags.KEY_UNIT -> LightWaight
        """
        event = Gst.Event.new_seek(
                playback_speed[self.settingwin.speed],
                Gst.Format.TIME,
                Gst.SeekFlags.FLUSH | Gst.SeekFlags.ACCURATE,
                Gst.SeekType.SET,
                location,
                Gst.SeekType.NONE,
                0)
        self.player.send_event(event)

    def clear(self):
        """
            State Non Playng
        """
        try:
            self.p_position = 0
            self.is_playing = False;
            if self.timer_id != -1:
                self.timer_id = -1
                GLib.source_remove(self.timer_id)
            if self.p_duration != Gst.CLOCK_TIME_NONE:
                self.player.set_state(Gst.State.READY)
                self.toolbox.seekadj.set_value(0)
                allsec = self.p_duration / 1000000000
                secv = "00:00/{0:02d}:{1:02d}".format((allsec / 60), (allsec % 60))
                self.statusbar.label[0].set_text(secv)
                self.toolbox.change_img(1)
            self.video_window.set_double_buffered(True)
            self.video_window.props.window.set_cursor(None)
        except Exception, e:
            self.messagebox(e)

    def change_video_size(self):
        """
            Video Window Size
        """
        if self.settingwin.video_size < 7:
            if self.uri == None: # First start and not set_size
                return
        self.set_focus(self.video_window)
        y = self.vsize.height
        # Vertical reference
        if self.settingwin.aspect == 0:
            asp = self.vsize.width
        elif self.settingwin.aspect == 1:
            asp = self.vsize.height * 4.0 / 3
        elif self.settingwin.aspect == 2:
            asp = self.vsize.height * 16.0 / 9
        elif self.settingwin.aspect == 3:
            asp = self.vsize.height * self.setting.aspect_rate[0] / self.setting.aspect_rate[1]
        # aspect rate change
        self.aframe.set(0.5, 0.5, float(asp)/float(y), False)
        # window size @ return
        if self.settingwin.video_size == 0:
            return
        x = int(asp)
        # calc
        menu_v = self.menubar.get_allocated_height()
        ctrl_v = self.toolbox.get_allocated_height()
        stat_v = self.statusbar.get_allocated_height()
        if self.settingwin.video_size == 2: # x2.0
            x *= 2
            y *= 2
        elif self.settingwin.video_size == 3: # x3.0
            x *= 3
            y *= 3
        elif self.settingwin.video_size == 4: # x0.5
            x /= 2
            y /= 2
        elif self.settingwin.video_size == 5: # x1.5
            x = x * 3 / 2
            y = y * 3 / 2
        elif self.settingwin.video_size == 6: # x2.5
            x = x * 5 / 2
            y = y * 5 / 2
        elif self.settingwin.video_size == 7: # set1
            x = self.setting.set_size[0][0]
            y = self.setting.set_size[0][1]
        elif self.settingwin.video_size == 8: # set2
            x = self.setting.set_size[1][0]
            y = self.setting.set_size[1][1]
        elif self.settingwin.video_size == 9: # set3
            x = self.setting.set_size[2][0]
            y = self.setting.set_size[2][1]
        if x ==0 or y == 0:
            return
        # Add menu and controls size
        if self.listbox.props.visible:
            x += self.listbox.props.width_request
        y += menu_v + ctrl_v + stat_v
        # Compare Current Size
        cx, cy = self.get_size()
        if cx != x or cy != y:
            self.resize(x, y)
        # Back to WindowSize
        if self.setting.size_win_only:
            self.settingwin.video_size = 0
            self.uimanager.get_widget("/MenuBar/View/x00").set_active(True)
            self.statusbar.label[1].set_text(size_str[0])

    def listup_change(self):
        """
            FileList Re-reading
        """
        if self.uri == None:
            return
        t = GLib.filename_from_uri(self.uri, None)
        dirc = GLib.path_get_dirname(t)
        self.path.name = GLib.path_get_basename(t)
        if self.path.dirc != dirc or self.listbox.is_text(self.path.name) == False:
            self.path.dirc = dirc
            self.listbox.liststore.clear()
            self.rlist.clear()
            files = numsort.get_file_numsort_list(self.path.dirc)
            for f in files:
                name = GLib.build_filenamev([self.path.dirc, f])
                gvfs = Gio.File.new_for_path(name)
                info = gvfs.query_info(
                        Gio.FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                        Gio.FileQueryInfoFlags.NONE,
                        None)
                mime = info.get_content_type()
                # Add Match MIME Type and Drop File
                if mime in self.mime_list or f == self.path.name:
                    it = self.listbox.liststore.append([f])
                    self.rlist.append(f)
                    # select change
                    if f == self.path.name:
                        self.listbox.select_text(self.path.name)
        else:
            # if Same Directory File Drop
            self.listbox.select_text(self.path.name)
        self.set_title("{0} ({1}/{2}) - Y901x".format(
                self.path.name,
                self.listbox.get_select_num() + 1,
                len(self.listbox.liststore))
            )
        # Create Random List
        self.rlist.shuffle()
        self.listbox.selection_scroll()

    def rename(self):
        """
            Rename the Playng File
        """
        if self.path.name == "":
            return
        label = Gtk.Label(self.path.name)
        entry = Gtk.Entry()
        entry.set_text(self.path.name)
        d = Gtk.Dialog( "リネーム",
                        self,
                        Gtk.DialogFlags.MODAL,
                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                        Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT) )
        try:
            d.vbox.pack_start(label, False, True, 0)
            d.vbox.pack_start(entry, False, True, 0)
            d.show_all()
            def dlg_ok(self):
                d.response(Gtk.ResponseType.ACCEPT)
            def on_focus_in(self, widget):
                try:
                    # calc the UNICODE String
                    s = unicode(entry.get_text())
                    i = s.rindex(".")
                    entry.select_region(0, i)
                except ValueError:
                    pass # is SelectAll
            def on_focus_out(self, widget):
                entry.select_region(0, 0)
            entry.connect("activate", dlg_ok)
            entry.connect("focus-in-event", on_focus_in)
            entry.connect("focus-out-event", on_focus_out)
            # Playing?
            is_play = False
            if self.player.get_state(1)[1] == Gst.State.PLAYING:
                is_play = True
                self.player.set_state(Gst.State.PAUSED)
            # Until the Success or Cancel.
            while 1:
                if d.run() == Gtk.RESPONSE_ACCEPT:
                    text = entry.get_text()
                    if text == self.path.name:
                        self.messagebox("変更されていません")
                    else:
                        if  text in os.listdir(self.path.dirc):
                            self.messagebox("同一ファイル名が見つかりました")
                        else:
                            newname = GLib.build_filenamev([self.path.dirc, text])
                            self.clear()
                            while Gtk.events_pending():
                                Gtk.main_iteration()
                            os.rename(GLib.build_filenamev([self.path.dirc, self.path.name]), newname)
                            while Gtk.events_pending():
                                Gtk.main_iteration()
                            self.set_uri("file://" + newname)
                            return
                else:
                    break
        finally:
            d.destroy()
        if is_play:
            self.player.set_state(Gst.State.PLAYING)

    def trash(self):
        """
            Trash the Playng File
        """
        if self.path.name == "":
            return
        # Playing
        is_play = False
        if self.player.get_state(1)[1] == Gst.State.PLAYING:
            is_play = True
            self.player.set_state(Gst.State.PAUSED)
        r = self.messagebox_trash("再生中ファイルをゴミ箱に捨てます\nよろしいですか？")
        if r == Gtk.RESPONSE_OK:
            fname = self.get_next_list()
            newname = GLib.build_filenamev([self.path.dirc, fname])
            self.clear()
            while Gtk.events_pending():
                Gtk.main_iteration()
            # Trash
            obj = Gio.File.new_for_path(GLib.build_filenamev([self.path.dirc, self.path.name]))
            try:
                res = obj.trash()
            except Exception, s:
                self.messagebox("{0|".format(s))
                res = False
            if res:
                while Gtk.events_pending():
                    Gtk.main_iteration()
                self.path.dirc = ""
                if fname != "":
                    self.set_uri("file://" + newname)
                else:
                    self.path.name = ""
                    self.listbox.liststore.clear()
                return
        if is_play:
            self.player.set_state(Gst.State.PLAYING)

    def trash_next(self):
        """
            Trash the Playng Under File
        """
        if self.path.name == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            it2 = model.iter_next(it)
            if it2:
                fname = model.get_value(it2, 0)
                delname = GLib.build_filenamev([self.path.dirc, fname])
                obj = Gio.File.new_for_path(delname)
                try:
                    res = obj.trash()
                    self.path.dirc = ""
                    self.listup_change()
                except Exception, s:
                    self.messagebox("{0}".format(s))

    def trash_prev(self):
        """
            Trash the Playng Upper File
        """
        if self.path.name == "":
            return
        model, it = self.listbox.selection.get_selected()
        if it:
            row = model.get_path(it)[0] - 1
            if row > -1:
                it2 = model.get_iter((row, ))
                fname = model.get_value(it2, 0)
                delname = GLib.build_filenamev([self.path.dirc, fname])
                obj = Gio.File.new_for_path(delname)
                try:
                    res = obj.trash()
                    self.path.dirc = ""
                    self.listup_change()
                except Exception, s:
                    self.messagebox("{0}".format(s))

    def get_next_list(self):
        """
            Listup Next Select Search
        """
        s = ""
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return s
        # next
        it2 = model.iter_next(it)
        if it2 == None:
            # no next
            row = model.get_path(it)[0] - 1
            if row < 0:
                return s
            self.listbox.selection.select_path(row)
            it3 = self.listbox.selection.get_selected()[1]
            s = model.get_value(it3, 0)
        else:
            self.listbox.selection.select_iter(it2)
            s = model.get_value(it2, 0)
        return s

    def change_fullscreen(self):
        """
            Switching FullScreen
        """
        if self.path.name == "":
            return
        if self.fullscreen:
            self.fullscreen = False
            self.props.window.unfullscreen()
            self.fullobj.destroy()
            self.fullobj = None
            self.menubar.show()
            if self.full_list:
                self.listbox.show()
        else:
            self.fullscreen = True
            self.full_list = self.listbox.get_property("visible")
            if self.full_list:
                self.listbox.hide()
            self.menubar.hide()
            self.fullobj = CFullCtrl(
                    self.hbox_ctrl,
                    self.toolbox,
                    self.vbox_main,
                    self.statusbar
                )
            self.props.window.fullscreen()


########################################################################
#
#                   Menu & Button Signal Handlers
#
########################################################################


    def on_open(self, widget, event=None):
        dlg = Gtk.FileChooserDialog("ファイルを開く", self,
                                    Gtk.GtkFileChooserAction.OPEN,
                                    (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, 
                                    Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
        # Set MIME Types Filter
        filt = Gtk.FileFilter()
        filt.set_name("Support file")
        for mimestring in self.mime_list:
            filt.add_mime_type(mimestring)
        dlg.add_filter(filt)
        filt = Gtk.FileFilter()
        filt.set_name("All files")
        filt.add_pattern("*")
        dlg.add_filter(filt)
        # Set Open Directry
        if self.path.dirc == "":
            # search VIDEOS Directry
            sd = GLib.get_user_special_dir(GLib.UserDirectory.DIRECTORY_VIDEOS)
            if sd:
                dlg.set_current_folder(sd)
            else:
                dlg.set_current_folder(GLib.get_home_dir())
        else:
            dlg.set_current_folder(self.path.dirc)
        r = dlg.run()
        if r == Gtk.ResponseType.OK:
            self.set_uri("file://" + dlg.get_filename())
        dlg.destroy()

    def on_play(self, widget, event=None):
        if self.player.get_state(1)[1] == Gst.State.PLAYING:
            self.player.set_state(Gst.State.PAUSED)
        else:
            self.player.set_state(Gst.State.PLAYING)
            #if self.player.set_state(Gst.State.PLAYING) == Gst.State.CHANGE_FAILURE:
            #    self.messaagebox("再生できませんでした")

    def on_stop(self, widget, event=None):
        self.clear()

    def on_prev(self, widget, event=None):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
            else:
                self.set_uri("file://{0}/{1}".format(self.path.dirc, self.rlist.get()))
        else:
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            row = model.get_path(it)[0] - 1
            if row < 0:
                return
            # Change Listbox Select
            self.listbox.selection.select_path(row)
            it = self.listbox.selection.get_selected()[1]
            s = model.get_value(it, 0)
            self.set_uri("file://{0}/{1}".format(self.path.dirc, s))

    def on_next(self, widget, event=None):
        if self.settingwin.repeat == 3:
            n = len(self.rlist.l)
            if n == 0:
                return
            elif n == 1:
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
            else:
                self.set_uri("file://{0}/{1}".format(self.path.dirc, self.rlist.get()))
        else:
            # is endressloop ?
            model, it = self.listbox.selection.get_selected()
            if it == None:
                return
            it = model.iter_next(it)
            if it == None:
                if self.setting.endressloop:
                    it = model.get_iter_first()
                else:
                    return
            # Change Listbox Select
            self.listbox.selection.select_iter(it)
            s = model.get_value(it, 0)
            self.set_uri("file://{0}/{1}".format(self.path.dirc, s))

    def on_rep_change(self, action, current):
        num = action.get_current_value()
        self.settingwin.repeat = num
        self.on_rep_btn(None, num)

    def on_rep_btn(self, widget, data):
        # widget==None is Escape
        if widget == None:
            if data == 0:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(False)
            elif data == 3:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(True)
            elif data == 2:
                self.toolbox.one.set_active(False)
                self.toolbox.all.set_active(True)
                self.toolbox.rdm.set_active(False)
            elif data == 1:
                self.toolbox.one.set_active(True)
                self.toolbox.all.set_active(False)
                self.toolbox.rdm.set_active(False)
        else:
            if widget.get_active():
                if self.settingwin.repeat != data:
                    action = self.actiongroup.get_action(ac2_str[data])
                    action.set_current_value(data)
                    if data != 3:
                        self.toolbox.rdm.set_active(False)
                    if data != 2:
                        self.toolbox.all.set_active(False)
                    if data != 1:
                        self.toolbox.one.set_active(False)
            else:
                if self.settingwin.repeat == data:
                    action = self.actiongroup.get_action(ac2_str[0])
                    action.set_current_value(0)
                    self.toolbox.one.set_active(False)
                    self.toolbox.all.set_active(False)
                    self.toolbox.rdm.set_active(False)

    def on_asp_change(self, action, current):
        num = action.get_current_value()
        self.settingwin.aspect = num
        if num == 3:
            s = "{0}:{1}".format(self.setting.aspect_rate[0], self.setting.aspect_rate[1])
            self.statusbar.label[2].set_text(s)
        else:
            self.statusbar.label[2].set_text(asp_str[num])
        self.change_video_size()

    def on_speed_up(self, widget, event=None):
        if self.settingwin.speed < 11:
            n = self.settingwin.speed + 1
            self.set_play_speed(n)

    def on_speed_down(self, widget, event=None):
        if self.settingwin.speed > 0:
            n = self.settingwin.speed - 1
            self.set_play_speed(n)

    def on_speed_reset(self, widget, event=None):
        if self.settingwin.speed != 5:
            self.set_play_speed(5)

    def on_quit(self, widget, event=None):
        self.close()
        return True

    def on_full(self, widget, event=None):
        self.change_fullscreen()

    def on_size_change(self, action, current):
        # Release FullScreen
        if self.fullscreen:
            self.change_fullscreen()
        # Draw StatusBar Text
        num = action.get_current_value()
        self.settingwin.video_size =  num
        self.statusbar.label[1].set_text(size_str[num])
        self.change_video_size()

    def on_setting(self, widget, event=None):
        dlg = y901dialog.SettingDialog(self, self.setting)
        try:
            if dlg.run() == Gtk.ResponseType.ACCEPT:
                self.setting.position      = dlg.cbox_pos.get_active()
                self.setting.esc_exit      = dlg.cbox_esc.get_active()
                self.setting.mem_rep       = dlg.cbox_memrep.get_active()
                self.setting.mem_aspect    = dlg.cbox_memaspect.get_active()
                self.setting.endressloop   = dlg.cbox_loop.get_active()
                self.setting.size_win_only = dlg.cbox_only.get_active()
                self.setting.init_size_on  = dlg.cbox_size.get_active()
                self.setting.init_size_val = dlg.combo_size.get_active()
                self.setting.severity = int(dlg.sb_mouse.get_value())
                startiter = dlg.mimebuf.get_start_iter()
                enditer   = dlg.mimebuf.get_end_iter()
                mimes = dlg.mimebuf.get_text(startiter, enditer, True).replace("\n", "|")
                if mimes != self.setting.mimes:
                    self.setting.mimes = mimes
                    self.mime_list = mimes.split("|")
                    self.path.dirc = ""
                    self.listup_change()
                self.setting.set_size[0][0] = int(dlg.sb[0].get_value())
                self.setting.set_size[0][1] = int(dlg.sb[1].get_value())
                self.setting.set_size[1][0] = int(dlg.sb[2].get_value())
                self.setting.set_size[1][1] = int(dlg.sb[3].get_value())
                self.setting.set_size[2][0] = int(dlg.sb[4].get_value())
                self.setting.set_size[2][1] = int(dlg.sb[5].get_value())
                aspect = dlg.entry_aspect.get_text().split(":")
                self.setting.aspect_rate[0] = int(aspect[0])
                self.setting.aspect_rate[1] = int(aspect[1])
                if self.setting.init_size_val:
                    self.settingwin.video_size = 0
                    self.uimanager.get_widget("/MenuBar/View/x00").set_active(True)
                    self.statusbar.label[1].set_text(size_str[0])
        except Exception, s:
            self.messagebox("{0}".format(s))
        finally:
            dlg.destroy()
        self.set_focus(self.video_window)

    def on_help(self, widget, event=None):
        GLib.spawn_command_line_async("http://palepoli.skr.jp/y901x/index.php")

    def on_about(self, widget, event=None):
        w = Gtk.AboutDialog()
        w.set_transient_for(self)
        w.set_program_name("Y901x")
        w.set_version(VERSION)
        w.set_copyright("Copyright(C) 2009-2013 sasakima-nao")
        w.set_license("GPL")
        w.set_logo(self.y901icon)
        # comments
        w.set_comments("GTK+ Version {0}.{1}.{2}".format(
                Gtk.get_major_version(),
                Gtk.get_minor_version(),
                Gtk.get_micro_version() ))
        w.set_website("http://palepoli.skr.jp/")
        w.run()
        w.destroy()

    def on_list_button_clicked(self, widget, event=None):
        if self.fullscreen:
            return
        width = self.listbox.props.width_request
        cx, cy = self.get_size()
        if self.listbox.props.visible:
            self.listbox.hide()
            self.resize(cx - width, cy)
        else:
            self.listbox.show()
            self.resize(cx + width, cy)


########################################################################
#
#                   MainWindow Signal Handlers
#
########################################################################

    
    def on_button_press(self, widget, event, data=None):
        """
            Start Mouse Gesture and W Click
        """
        if event.button == 3:
            self.gesture.start = True
            self.gesture.x = event.x_root
            self.gesture.y = event.y_root
            self.gesture.result = ""
            self.gesture.prevchar = ""
            self.gesture.title = ""
        elif event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS:
            self.change_fullscreen()
    
    def on_motion_notify(self, widget, event, data=None):
        """
            Mouse Gesture
        """
        if self.gesture.start:
            x = abs(event.x_root - self.gesture.x)
            y = abs(event.y_root - self.gesture.y)
            s = ""
            if (x > self.setting.severity) or (y > self.setting.severity):
                if x > y:
                    if (self.gesture.x - event.x_root) > 0:
                        s = "←"
                    else:
                        s = "→"
                else:
                    if (self.gesture.y - event.y_root) > 0:
                        s = "↑"
                    else:
                        s = "↓"
                if self.gesture.prevchar != s:
                    self.gesture.prevchar = s
                    self.gesture.result += s
                self.gesture.x = event.x_root
                self.gesture.y = event.y_root
                if self.gesture.title == "":
                    self.gesture.title = self.get_title()
                self.set_title(self.gesture.result)
    
    def on_button_release(self, widget, event, data=None):
        """
            Mouse Gesture
            if event.button == 3: is no
        """
        if self.gesture.start:
            self.gesture.start = False
            # Separation (Prevent PopupMenu)
            if self.gesture.result == "" and self.get_title() != "":
                self.popup_menu.popup(None, None, None, 0, event.time)
            else:
                if self.gesture.title != "":
                    self.set_title(self.gesture.title)
                if self.gesture.result == "→":
                    self.on_next(widget,0)
                elif self.gesture.result == "←":
                    self.on_prev(widget,0)
                elif self.gesture.result == "↓→":
                    self.close()
        return True

    def on_drag_data_received(self, widget, drag_context, x, y, data, info, time):
        """
            D&D FileNames split CRLF or LF
        """
        drop = data.get_uris()[0]
        if "//" in drop:
            self.set_uri(drop)
        else:
            self.set_uri("file://" + drop)

    def on_key_press(self, widget, event, data=None):
        """
            event.state Mix Gdk.MOD2_MASK
        """
        if event.state == (Gdk.ModifierType.CONTROL_MASK | Gdk.ModifierType.MOD2_MASK):
            val = self.toolbox.volumebar.get_value()
            if event.keyval == Gdk.KEY_Up:
                if val < 100:
                    self.toolbox.volumebar.set_value(val + 1)
                return True
            elif event.keyval == Gdk.KEY_Down:
                if val > 0:
                    self.toolbox.volumebar.set_value(val - 1)
                return True
        else:
            if event.keyval == Gdk.KEY_Up:
                self.on_prev(widget)
                return True
            elif event.keyval == Gdk.KEY_Down:
                self.on_next(widget)
                return True

    def on_key_release(self, widget, event):
        self.set_focus(self.video_window)


########################################################################
#
#                       About Signal Handlers
#
########################################################################


    def put_time_status(self, location):
        sec = location // 1000000000
        allsec = self.p_duration // 1000000000
        if allsec >= 3600:
            s = time.strftime("%H:%M:%S", time.gmtime(sec))
            t = time.strftime("%H:%M:%S", time.gmtime(allsec))
        else:
            s = time.strftime("%M:%S", time.gmtime(sec))
            t = time.strftime("%M:%S", time.gmtime(allsec))
        self.statusbar.label[0].set_text("{0}/{1}".format(s, t))

    def on_timer(self):
        if self.timer_id == -1:
            return
        try:
            self.p_position = self.player.query_position(Gst.Format.TIME)[1]  # 1.0
        except:
            self.p_position = Gst.CLOCK_TIME_NONE
        try:
            # Change SeekBar and Statusbar Time
            if self.p_position != Gst.CLOCK_TIME_NONE:
                self.toolbox.seekadj.set_value(self.p_position)
                self.put_time_status(self.p_position)
            # GetCursolPos
            a, x, y, b = Gdk.Display.get_default().get_pointer()
            # FullScreen at Controler
            if self.fullobj:
                if self.fullobj.y_pos < y:
                    self.fullobj.show_all()
                else:
                    self.fullobj.hide()
            # Cursor Auto Invisible
            if self.mousepoint.x == x and self.mousepoint.y == y:
                if self.invisiblecount < 5:
                    self.invisiblecount += 1
                else:
                    self.video_window.props.window.set_cursor(self.noncursor)
            else:
                self.invisiblecount = 0;
                self.mousepoint.x = x
                self.mousepoint.y = y
                self.video_window.props.window.set_cursor(None)
            # return True is Repeat
            return self.is_playing
        except Exception, e:
            self.messagebox(e)

    def on_seek_value_changed(self, widget, event=None):
        if self.timer_id == -1:
            return
        if self.seekinfo.is_seek:
            location = widget.get_value()
            self.set_play_position(location)
            self.put_time_status(location)

    def on_seek_button_press_event(self, widget, event):
        if self.timer_id == -1:
            return
        if event.button < 4:
            if self.player.get_state(1)[1] == Gst.State.PLAYING:
                self.player.set_state(Gst.State.PAUSED)
                self.seekinfo.playing = True
            GLib.source_remove(self.timer_id)
            self.seekinfo.is_seek = True
            self.seekinfo.button = event.button

    def on_seek_button_release_event(self, widget, event):
        self.set_focus(self.video_window)
        if self.timer_id == -1:
            return
        self.seekinfo.is_seek = False
        if self.seekinfo.playing:
            self.seekinfo.playing = False
            self.player.set_state(Gst.State.PLAYING)
        self.timer_id = GLib.timeout_add(200, self.on_timer)

    def set_volume(self, value):
        if self.player.get_volume(GstAudio.StreamVolumeFormat.CUBIC) != value:
            self.player.set_volume(GstAudio.StreamVolumeFormat.CUBIC, value)

    def on_volume_value_changed(self, widget, event=None):
        self.set_volume(widget.get_value() / 100.0)

    def on_volume_button_release_event(self, widget, event):
        self.set_focus(self.video_window)

    def on_listbox_button_press(self, widget, event=None):
        if event.button == 3:
            self.listpopup.popup(None, None, None, 0, event.time)
            return True

    def on_listbox_button_release(self, widget, event=None):
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return
        s = model.get_value(it, 0)
        if s != self.path.name:
            new_uri = "file://" + self.path.dirc + "/" + s
            self.set_uri(new_uri)
        self.set_focus(self.video_window)

    def on_aframe_draw(self, widget, cr):
        """
            GtkAspectFrame FillRect
        """
        width = widget.get_allocated_width()
        height = widget.get_allocated_height()
        cr.set_source_rgb(0.0, 0.0, 0.0)
        cr.paint()

    def on_keypad(self, accelGroup, window, keyval, modifier):
        """
            processing ten key
        """
        g = None
        if keyval == Gdk.KEY_KP_0:
            g = self.uimanager.get_widget("/MenuBar/View/x00")
        elif keyval == Gdk.KEY_KP_1:
            g = self.uimanager.get_widget("/MenuBar/View/x10")
        elif keyval == Gdk.KEY_KP_2:
            g = self.uimanager.get_widget("/MenuBar/View/x20")
        elif keyval == Gdk.KEY_KP_3:
            g = self.uimanager.get_widget("/MenuBar/View/x30")
        elif keyval == Gdk.KEY_KP_4:
            g = self.uimanager.get_widget("/MenuBar/View/x05")
        elif keyval == Gdk.KEY_KP_5:
            g = self.uimanager.get_widget("/MenuBar/View/x15")
        elif keyval == Gdk.KEY_KP_6:
            g = self.uimanager.get_widget("/MenuBar/View/x25")
        elif keyval == Gdk.KEY_KP_7:
            g = self.uimanager.get_widget("/MenuBar/View/xs1")
        elif keyval == Gdk.KEY_KP_8:
            g = self.uimanager.get_widget("/MenuBar/View/xs2")
        elif keyval == Gdk.KEY_KP_9:
            g = self.uimanager.get_widget("/MenuBar/View/xs3")
        # exec on_size_change signal
        g.set_active(True)

    def on_keydown(self, accelGroup, window, keyval, modifier):
        """
            processing Non Menu Keybord
        """
        if keyval == Gdk.KEY_F12:
            self.on_list_button_clicked(window)
        elif keyval == Gdk.KEY_F5:
            self.path.dirc = ""
            self.listup_change()
        elif keyval == Gdk.KEY_F2:
            self.rename()
        elif keyval == Gdk.KEY_Escape:
            if self.fullscreen:
                self.change_fullscreen()
            elif self.setting.esc_exit:
                self.close()
        elif keyval == Gdk.KEY_Delete:
            if modifier == Gdk.SHIFT_MASK:
                self.trash_prev()
            elif modifier == Gdk.CONTROL_MASK:
                self.trash_next()
            else:
                self.trash()
        elif keyval == Gdk.KEY_F11:
            self.change_fullscreen()
        elif keyval == Gdk.KEY_Return:
            self.on_play(window)
        elif keyval == Gdk.KEY_KP_Add:
            if self.settingwin.speed < 11:
                n = self.settingwin.speed + 1
                self.set_play_speed(n)
        elif keyval == Gdk.KEY_KP_Subtract:
            if self.settingwin.speed > 0:
                n = self.settingwin.speed - 1
                self.set_play_speed(n)
        elif keyval == Gdk.KEY_KP_Enter:
            if self.settingwin.speed != 5:
                self.set_play_speed(5)

    def on_soundmenu(self, action, current):
        """
            Multi Track Audio Select
        """
        n = action.get_current_value()
        self.player.set_property("current-audio", n)
        self.settingwin.n_audio = n


########################################################################
#
#                       GStreamer Event Handlers
#
########################################################################


    def gst_on_message(self, bus, message):
        """
            GStreamer Callback
        """
        t = message.type
        if t == Gst.MessageType.EOS:
            self.clear()
            # Repeat
            if self.settingwin.repeat > 1:
                #self.on_next(self)
                Gdk.threads_enter()
                self.next_menu.emit("activate")
                Gdk.threads_leave()
            elif self.settingwin.repeat == 1:
                #self.clear()
                self.set_play_position(0)
                self.player.set_state(Gst.State.PLAYING)
        elif t == Gst.MessageType.ERROR:
            self.clear()
            err, debug = message.parse_error()
            self.messagebox(str(err) + "\n" + str(debug))  
            self.toolbox.change_img(0)
        elif t == Gst.MessageType.STATE_CHANGED:
            if Gst.State.PLAYING == message.parse_state_changed()[1]:
                self.toolbox.change_img(0)
                # This is First Play
                if not self.is_playing:
                    try:
                        self.p_duration = self.player.query_duration(Gst.Format.TIME)[1]  # 1.0
                    except:
                        self.p_duration = Gst.CLOCK_TIME_NONE
                    if self.p_duration != Gst.CLOCK_TIME_NONE:
                        self.toolbox.seekadj.set_upper(self.p_duration)
                        self.is_playing = True
                        # toolbox Buttons Enabled
                        self.toolbox.set_enable_ctrl(True)
                        # Timer on
                        self.timer_id = GLib.timeout_add(200, self.on_timer)
                        # File Names Listup
                        self.listup_change()
                        # playbin2
                        # Audio
                        asink = self.player.get_property("audio-sink")
                        if asink:
                            self.audiomenu.remove_menu()
                            # Track num
                            n_audio = self.player.get_property("n-audio")
                            if n_audio == 1:
                                self.player.set_property("current-audio", 0)
                            else:
                                actions = []
                                x = ""
                                for i in range(n_audio):
                                    a = "sound{0}".format(i)
                                    s = "トラック #{0}".format(i)
                                    # GtkRadioActionEntry
                                    # name, stock_id, label, accelerator, tooltip, value
                                    t = (a, None, s, None, s, i)
                                    actions.append(t)
                                    x += '<menuitem action="{0}"/>'.format(a)
                                xml = audio_str.format(x)
                                # Add Menu
                                self.audiomenu.add_menu(actions, xml, self.on_soundmenu, self.settingwin.n_audio)
                                self.player.set_property("current-audio", self.settingwin.n_audio)
                            self.toolbox.volumebar.set_property("sensitive", True)
                            v = self.toolbox.volumeadj.get_value() / 100.0
                            self.set_volume(v)
                        else:
                            self.toolbox.volumebar.set_property("sensitive", False)
                        # Video vsink == None @ Music File
                        vsink = self.player.get_property("video-sink")
                        if vsink:
                            for pad in vsink.pads:
                                # W Buffer Off
                                self.video_window.set_double_buffered(False)
                                caps = pad.get_current_caps()
                                #(success, num, denom) = caps.get_structure(0).get_fraction('framerate')
                                success, width = caps.get_structure(0).get_int("width")
                                success, height = caps.get_structure(0).get_int("height")
                                self.vsize.width = width
                                self.vsize.height = height
                                #
                                self.change_video_size()
                                aspect_rate = float(self.vsize.width) / float(self.vsize.height)
                                self.aframe.set(0.5, 0.5, aspect_rate, False)
                                #self.change_video_size()
                        self.set_play_position(self.settingwin.speed)
            else:
                self.toolbox.change_img(1)

    def gst_on_sync_message(self, bus, message):
        """
            GStreamer Sync Func
        """
        if message.get_structure().get_name() == "prepare-window-handle":
            xid = self.video_window.props.window.get_xid()
            imagesink = message.src
            imagesink.props.force_aspect_ratio = False
            imagesink.set_window_handle(xid)


########################################################################
#
#                               main
#
########################################################################


def main():
    if "-V" in sys.argv:
        print(VERSION)
        return 0
    GObject.threads_init()
    Gst.init(None)
    w = Y901window()
    # argv processing
    if len(sys.argv) > 1:
        if "//" in sys.argv[1]:
            w.set_uri(sys.argv[1])
        else:
            w.set_uri("file://" + os.path.abspath(sys.argv[1]))
    Gtk.main()

if __name__ == "__main__":
    main()
